@@ -1,6 +1,7 @@
 import collections
 import itertools
 import os
+import sys
 
 import astroid
 import astroid.builder
@@ -9,6 +10,14 @@ import sphinx.util.docstrings
 from . import _astroid_utils
 
 
+if sys.version_info < (3, 10):  # PY310
+    from stdlib_list import in_stdlib
+else:
+
+    def in_stdlib(module_name: str) -> bool:
+        return module_name in sys.stdlib_module_names
+
+
 def _prepare_docstring(doc):
     return "\n".join(sphinx.util.docstrings.prepare_docstring(doc))
 
@@ -135,27 +144,28 @@ class Parser:
         for base in itertools.chain(iter((node,)), node.ancestors()):
             seen = set()
             base_children = []
-            if base.qname() in (
-                "__builtins__.object",
-                "builtins.object",
-                "builtins.type",
-            ):
-                continue
-            for child in base.get_children():
-                name = getattr(child, "name", None)
-                if isinstance(child, (astroid.Assign, astroid.AnnAssign)):
-                    assign_value = _astroid_utils.get_assign_value(child)
-                    if not assign_value:
-                        continue
-                    name = assign_value[0]
 
-                if not name or name in overridden:
-                    continue
-                seen.add(name)
-                child_data = self.parse(child)
-                base_children.extend(
-                    _parse_child(node, child_data, overloads, base, name)
-                )
+            # Don't document members inherited from standard library classes
+            # unless that class is abstract.
+            base_module = base.qname().split(".", 1)[0]
+            if in_stdlib(base_module) and not _astroid_utils.is_abstract_class(base):
+                continue
+
+            for child in base.get_children():
+                children_data = self.parse(child)
+                for child_data in children_data:
+                    name = child_data["name"]
+
+                    existing_child = children.get(name)
+                    if existing_child and not existing_child["doc"]:
+                        existing_child["doc"] = child_data["doc"]
+
+                    if name in overridden:
+                        continue
+
+                    seen.add(name)
+                    if _parse_child(node, child_data, overloads, base):
+                        base_children.append(child_data)
 
             overridden.update(seen)
 
@@ -287,11 +297,13 @@ class Parser:
         top_name = node.name.split(".", 1)[0]
         for child in node.get_children():
             if _astroid_utils.is_local_import_from(child, top_name):
-                child_data = self._parse_local_import_from(child)
+                children_data = self._parse_local_import_from(child)
             else:
-                child_data = self.parse(child)
+                children_data = self.parse(child)
 
-            data["children"].extend(_parse_child(node, child_data, overloads))
+            for child_data in children_data:
+                if _parse_child(node, child_data, overloads):
+                    data["children"].append(child_data)
 
         return data
 
@@ -325,7 +337,7 @@ class Parser:
         return [data]
 
     def parse(self, node):
-        data = {}
+        data = []
 
         node_type = node.__class__.__name__.lower()
         parse_func = getattr(self, "parse_" + node_type, None)
@@ -336,28 +348,28 @@ class Parser:
                 data = self.parse(child)
                 if data:
                     break
+
         return data
 
 
-def _parse_child(node, child_data, overloads, base=None, name=None):
-    result = []
-    for single_data in child_data:
-        if single_data["type"] in ("function", "method", "property"):
-            if name is None:
-                name = single_data["name"]
-            if name in overloads:
-                grouped = overloads[name]
-                grouped["doc"] = single_data["doc"]
-                if single_data["is_overload"]:
-                    grouped["overloads"].append(
-                        (single_data["args"], single_data["return_annotation"])
-                    )
-                continue
-            if single_data["is_overload"] and name not in overloads:
-                overloads[name] = single_data
+def _parse_child(node, child_data, overloads, base=None) -> bool:
+    if child_data["type"] in ("function", "method", "property"):
+        name = child_data["name"]
+        if name in overloads:
+            grouped = overloads[name]
+            grouped["doc"] = child_data["doc"]
 
-        if base:
-            single_data["inherited"] = base is not node
-        result.append(single_data)
+            if child_data["is_overload"]:
+                grouped["overloads"].append(
+                    (child_data["args"], child_data["return_annotation"])
+                )
 
-    return result
+            return False
+
+        if child_data["is_overload"] and name not in overloads:
+            overloads[name] = child_data
+
+    if base:
+        child_data["inherited"] = base is not node
+
+    return True
